@@ -67,6 +67,8 @@
 #define OPT_TYPING      0x00000100 /* Some pieces of code make assumptions */
 #define OPT_THINKING    0x00000200 /* about these values... Stupid me! */
 #define OPT_NOOTR       0x00001000 /* protocol not suitable for OTR */
+#define OPT_PONGS       0x00010000 /* Service sends us keep-alives */
+#define OPT_PONGED      0x00020000 /* Received a keep-alive during last interval */
 
 /* ok. now the fun begins. first we create a connection structure */
 struct im_connection
@@ -87,11 +89,8 @@ struct im_connection
 	GSList *deny;
 	int permdeny;
 	
-	char displayname[128];
 	char *away;
 	
-	int evil;
-	
 	/* BitlBee */
 	bee_t *bee;
 	
@@ -126,7 +125,6 @@ struct buddy {
 	char name[80];
 	char show[BUDDY_ALIAS_MAXLEN];
 	int present;
-	int evil;
 	time_t signon;
 	time_t idle;
 	int uc;
@@ -197,7 +195,7 @@ struct prpl {
 	void (* add_deny)	(struct im_connection *, char *who);
 	void (* rem_permit)	(struct im_connection *, char *who);
 	void (* rem_deny)	(struct im_connection *, char *who);
-	/* Doesn't actually have UI hooks. */
+	/* Doesn't actually have UI hooks. Not used at all, can be removed. */
 	void (* set_permit_deny)(struct im_connection *);
 	
 	/* Request profile info. Free-formatted stuff, the IM module gives back
