@@ -569,10 +569,6 @@ void ext_yahoo_got_buddies( int id, YList *buds )
 	}
 }
 
-void ext_yahoo_got_ignore( int id, YList *igns )
-{
-}
-
 void ext_yahoo_got_identities( int id, YList *ids )
 {
 }
@@ -813,13 +809,6 @@ void ext_yahoo_got_buddy_change_group( int id, const char *me, const char *who,
 	imcb_add_buddy( ic, who, new_group );
 }
 
-/* Because we don't want asynchronous connects in BitlBee, and because
-   libyahoo doesn't seem to use this one anyway, this one is now defunct. */
-int ext_yahoo_connect(const char *host, int port)
-{
-	return -1;
-}
-
 static void byahoo_accept_conf( void *data )
 {
 	struct byahoo_conf_invitation *inv = data;
